using System;
using System.Linq;
using System.Collections.ObjectModel;
using CODE.Framework.Wpf.Mvvm;

namespace DocumentExamples.Models.Print
{
    public class InvoiceViewModel : ViewModel
    {
        private decimal _taxRate;
        private decimal _taxAmount;
        private decimal _total;

        public InvoiceViewModel()
        {
            LineItems = new ObservableCollection<LineItemViewModel>
                {
                    new LineItemViewModel(this)
                        {
                            Quantity = 2,
                            Description = "Quad Wave windsurf board with accessories",
                            Price = 2490m,
                            ImageUrl = "http://d2z2i12dpvcgkc.cloudfront.net/wp-content/uploads/2012/09/boards_2013_custom_rendering.jpg"
                        },
                    new LineItemViewModel(this)
                        {
                            Quantity = 1,
                            Description = "Wave & Ride windsurf board with accessories",
                            Price = 1990m,
                            ImageUrl = "http://d2z2i12dpvcgkc.cloudfront.net/wp-content/uploads/2012/09/boards_2013_one_pro_render_2.jpg"
                        },
                    new LineItemViewModel(this)
                        {
                            Quantity = 1,
                            Description = "Free-Style windsurf board with accessories",
                            Price = 2290m,
                            ImageUrl = "http://d2z2i12dpvcgkc.cloudfront.net/wp-content/uploads/2012/09/boards-2013-air-render.jpg"
                        },
                };

            InvoiceDate = DateTime.Now;
            CustomerName = "Markus Egger";
            CustomerAddress = "6605 Cypresswood Dr.\r\nSuite 300\r\nSpring, TX 77379";
            TaxRate = 4m;

            Actions.Add(new CloseCurrentViewAction(this, beginGroup: true));

            RecalculateTotal();
        }

        public string CustomerName { get; set; }
        public string CustomerAddress { get; set; }

        public DateTime InvoiceDate { get; set; }

        public decimal TaxRate
        {
            get { return _taxRate; }
            set
            {
                _taxRate = value;
                NotifyChanged("TaxRate");
                RecalculateTotal();
            }
        }

        public decimal TaxAmount
        {
            get { return _taxAmount; }
            set
            {
                _taxAmount = value;
                NotifyChanged("TaxAmount");
                NotifyChanged("TaxAmountDisplay");
            }
        }

        public string TaxRateDisplay
        {
            get { return TaxRate.ToString("C"); }
        }
        public string TaxAmountDisplay
        {
            get { return TaxAmount.ToString("C"); }
        }

        public decimal Total
        {
            get { return _total; }
            set
            {
                _total = value;
                NotifyChanged("Total");
                NotifyChanged("TotalDisplay");
            }
        }

        public string TotalDisplay
        {
            get { return Total.ToString("C"); }
        }

        public ObservableCollection<LineItemViewModel> LineItems { get; set; }

        public void RecalculateTotal()
        {
            if (LineItems == null) return;
            var total = LineItems.Sum(l => l.ItemTotal);
            var tax = total/100*TaxRate;
            TaxAmount = tax;
            Total = total + tax;
        }
    }

    public class LineItemViewModel : ViewModel
    {
        private readonly InvoiceViewModel _parent;

        public LineItemViewModel(InvoiceViewModel parent)
        {
            _parent = parent;
        }

        private decimal _quantity;
        private string _description;
        private string _imageUrl;
        private decimal _price;

        public decimal Quantity
        {
            get { return _quantity; }
            set 
            { 
                _quantity = value; 
                NotifyChanged("Quantity");
                NotifyChanged("Price");
                NotifyChanged("PriceDisplay");
                NotifyChanged("ItemTotal");
                NotifyChanged("ItemTotalDisplay");
                if (_parent != null)
                    _parent.RecalculateTotal();
            }
        }

        public string Description
        {
            get { return _description; }
            set
            {
                _description = value;
                NotifyChanged("Description");
            }
        }

        public string ImageUrl
        {
            get { return _imageUrl; }
            set
            {
                _imageUrl = value;
                NotifyChanged("ImageUrl");
            }
        }

        public decimal Price
        {
            get { return _price; }
            set
            {
                _price = value;
                NotifyChanged("Price");
                NotifyChanged("PriceDisplay");
                NotifyChanged("ItemTotal");
                NotifyChanged("ItemTotalDisplay");
                if (_parent != null)
                    _parent.RecalculateTotal();
            }
        }

        public string PriceDisplay
        {
            get { return Price.ToString("C"); }
        }

        public decimal ItemTotal
        {
            get { return Quantity * Price; }
        }

        public string ItemTotalDisplay
        {
            get { return ItemTotal.ToString("C"); }
        }
    }
}
