using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using CODE.Framework.Wpf.Documents;
using CODE.Framework.Wpf.Mvvm;

namespace DocumentExamples.Models.Print
{
    public class OrdersViewModel : ViewModel
    {
        public OrdersViewModel()
        {
            Months = new ObservableCollection<OrdersByMonthViewModel>();

            for (var month = 1; month < 13; month++)
                Months.Add(new OrdersByMonthViewModel(month, 2012));

            Actions.Add(new ViewAction("Save", significance: ViewActionSignificance.AboveNormal, brushResourceKey: "CODE.Framework-Icon-Save", execute: (a, o) =>
                {
                    var doc = Controller.GetDocumentForModel(this);
                    if (doc == null) return;
                    doc.SaveAsXps(@"d:\Test.xps");
                    Process.Start(@"d:\Test.xps");
                }));
            Actions.Add(new ViewAction("Print", significance: ViewActionSignificance.AboveNormal, brushResourceKey: "CODE.Framework-Icon-Print", execute: (a, o) =>
                {
                    var doc = Controller.GetDocumentForModel(this);
                    if (doc == null) return;
                    doc.Print();
                }));
            Actions.Add(new CloseCurrentViewAction(this, beginGroup: true) {Significance = ViewActionSignificance.AboveNormal});

            Controller.Status("Orders loaded.");
        }

        public ObservableCollection<OrdersByMonthViewModel> Months { get; set; }
    }

    public class OrdersByMonthViewModel : ViewModel
    {
        public OrdersByMonthViewModel(int month, int year)
        {
            Orders = new ObservableCollection<OrderViewModel>();

            var startDate = new DateTime(year, month, 1);

            Title = "Orders for " + startDate.ToString("MMMM") + " " + year;

            var rand = new Random(month);
            var count = rand.Next(5, 15);

            for (var counter = 0; counter < count; counter++)
                Orders.Add(new OrderViewModel
                    {
                        CustomerName = "Example Customer #" + counter,
                        OrderDate = new DateTime(year, month, (counter+1) * 2),
                        Total = rand.Next(5000, 100000)
                    });
        }

        public string Title { get; set; }

        public decimal TotalAmount
        {
            get { return Orders.Sum(o => o.Total); }
        }

        public string TotalAmountDisplay
        {
            get { return TotalAmount.ToString("C"); }
        }

        public decimal AverageAmount
        {
            get { return Orders.Average(o => o.Total); }
        }

        public string AverageAmountDisplay
        {
            get { return AverageAmount.ToString("C"); }
        }

        public ObservableCollection<OrderViewModel> Orders { get; set; }
    }

    public class OrderViewModel : ViewModel
    {
        public string CustomerName { get; set; }
        public DateTime OrderDate { get; set; }
        public string OrderDateDisplay
        {
            get { return OrderDate.ToShortDateString(); }
        }
        public decimal Total { get; set; }
        public string TotalDisplay
        {
            get { return Total.ToString("C"); }
        }
    }
}